//  Douglas M. Hurst
//  8912 Spur Road
//  Springfield, VA 22153
//  CompuServe 72767,1163
//
//  BAR.C - Is a library of functions designed to allow printing
//  of 3 of 9 and USPS zip codes on HP LaserJet and compatible
//  printers WITHOUT THE REQUIREMENT FOR A FONT CARTRIDGE.
//
//  See the test program BARPRINT.C for more documentation.

#include "bar.h"

// H P _ P R I N T _ B A R 3 9
//
int hp_printbar39(FILE *fp, int col, int row, char *code)
{
  char *code_plus,bcode[11];
  int i,j,pointx,pointy;

  // ALLOCATE SPACE TO HOLD THE ASTERISKS
  code_plus = (char *)malloc(strlen(code)+3);
  strcpy(code_plus,code);

  // NOW ADD THE ASTERISKS
  add_asterisks(code_plus);

  // LOCATE THE PRINTER CURSOR
  hp_gotoxy(fp, col, row, &pointx, &pointy);

  // NOW, PRINT THE BARCODE
  for (i=0 ; code_plus[i] ; i++)
  {
    switch(code_plus[i])
    {
      case '$': strcpy(bcode,"-1-1-1----"); break;
      case '%': strcpy(bcode,"---1-1-1--"); break;
      case '*': strcpy(bcode,"-1--1-1---"); break;
      case '+': strcpy(bcode,"-1---1-1--"); break;
      case ',': strcpy(bcode,"-11---1---"); break;
      case '-': strcpy(bcode,"-1----1-1-"); break;
      case '.': strcpy(bcode,"11----1---"); break;
      case '/': strcpy(bcode,"-1-1---1--"); break;
      case '0': strcpy(bcode,"---11-1---"); break;
      case '1': strcpy(bcode,"1--1----1-"); break;
      case '2': strcpy(bcode,"--11----1-"); break;
      case '3': strcpy(bcode,"1-11------"); break;
      case '4': strcpy(bcode,"---11---1-"); break;
      case '5': strcpy(bcode,"1--11-----"); break;
      case '6': strcpy(bcode,"--111-----"); break;
      case '7': strcpy(bcode,"---1--1-1-"); break;
      case '8': strcpy(bcode,"1--1--1---"); break;
      case '9': strcpy(bcode,"--11--1---"); break;
      case 'A': strcpy(bcode,"1----1--1-"); break;
      case 'B': strcpy(bcode,"--1--1--1-"); break;
      case 'C': strcpy(bcode,"1-1--1----"); break;
      case 'D': strcpy(bcode,"----11--1-"); break;
      case 'E': strcpy(bcode,"1---11----"); break;
      case 'F': strcpy(bcode,"--1-11----"); break;
      case 'G': strcpy(bcode,"-----11-1-"); break;
      case 'H': strcpy(bcode,"1----11---"); break;
      case 'I': strcpy(bcode,"--1--11---"); break;
      case 'J': strcpy(bcode,"----111---"); break;
      case 'K': strcpy(bcode,"1------11-"); break;
      case 'L': strcpy(bcode,"--1----11-"); break;
      case 'M': strcpy(bcode,"1-1----1--"); break;
      case 'N': strcpy(bcode,"----1--11-"); break;
      case 'O': strcpy(bcode,"1---1--1--"); break;
      case 'P': strcpy(bcode,"--1-1--1--"); break;
      case 'Q': strcpy(bcode,"------111-"); break;
      case 'R': strcpy(bcode,"1-----11--"); break;
      case 'S': strcpy(bcode,"--1---11--"); break;
      case 'T': strcpy(bcode,"----1-11--"); break;
      case 'U': strcpy(bcode,"11------1-"); break;
      case 'V': strcpy(bcode,"-11-----1-"); break;
      case 'W': strcpy(bcode,"111-------"); break;
      case 'X': strcpy(bcode,"-1--1---1-"); break;
      case 'Y': strcpy(bcode,"11--1-----"); break;
      case 'Z': strcpy(bcode,"-11-1-----"); break;
      default:  free(code_plus); return(1);
    }
    for (j=0 ; bcode[j] ; j++)
    {
      if (j % 2 == 0)
      {
        if (bcode[j] == '1')
        {
          fprintf(fp,BAR,pointx,pointy,BAR_WIDE,BAR_TALL,BLACK);
          pointx+=9;
        }
        else
        {
          fprintf(fp,BAR,pointx,pointy,BAR_NARROW,BAR_TALL,BLACK);
          pointx+=3;
        }
      }
      else
      {
        if (bcode[j] == '1')
        {
          fprintf(fp,BAR,pointx,pointy,BAR_WIDE,BAR_TALL,WHITE);
          pointx+=9;
        }
        else
        {
          fprintf(fp,BAR,pointx,pointy,BAR_NARROW,BAR_TALL,WHITE);
          pointx+=3;
        }
      }
    }
  }

  // TIDY UP
  free(code_plus); 
  return(0);
}

// H P _ P R I N T B A R Z I P
//
int hp_printbarzip(FILE *fp, int col, int row, char *code, char *addr)
{
  char *code_plus,bcode[11];
  int i,j,pointx,pointy;

  // ALLOCATE SPACE TO HOLD THE ASTERISKS AND CHECK DIGIT
  code_plus = (char *)malloc(strlen(code)+5);
  strcpy(code_plus,code);

  // ADD THE ASTERISKS AND CHECKDIGIT
  add_adv_ZIP_check_digit(code_plus,addr);

  // LOCATE THE PRINTER CURSOR
  hp_gotoxy(fp,col,row,&pointx,&pointy);


  // NOW, PRINT THE BARCODE
  for (i=0 ; code_plus[i] ; i++)
  {
    switch(code_plus[i])
    {
      case '*': strcpy(bcode,"1-"); break;
      case '0': strcpy(bcode,"1-1-0-0-0-"); break;
      case '1': strcpy(bcode,"0-0-0-1-1-"); break;
      case '2': strcpy(bcode,"0-0-1-0-1-"); break;
      case '3': strcpy(bcode,"0-0-1-1-0-"); break;
      case '4': strcpy(bcode,"0-1-0-0-1-"); break;
      case '5': strcpy(bcode,"0-1-0-1-0-"); break;
      case '6': strcpy(bcode,"0-1-1-0-0-"); break;
      case '7': strcpy(bcode,"1-0-0-0-1-"); break;
      case '8': strcpy(bcode,"1-0-0-1-0-"); break;
      case '9': strcpy(bcode,"1-0-1-0-0-"); break;
      case '|': strcpy(bcode,"1-"); break;
      default:  free(code_plus); return(1);
    }
    for (j=0 ; bcode[j] ; j++)
    {
      if (j % 2 == 0)
      {
        if (bcode[j] == '1')
          fprintf(fp,BAR,pointx,pointy,ZIP_NARROW,ZIP_TALL,BLACK);
        else
          fprintf(fp,BAR,pointx,pointy+ZIP_SHORT_START,ZIP_NARROW,ZIP_SHORT,BLACK);
      }
      else
        fprintf(fp,BAR,pointx,pointy+ZIP_SHORT_START,ZIP_WIDE,ZIP_SHORT,WHITE);
      pointx += 7;
    }
  } 

  // TIDY UP
  free(code_plus);
  return(0);
}

// A D D _ A D V _ Z I P _ C H E C K _ D I G I T
//
// The ZIP code should be passed as a string of solid numeric characters.
// It may be 5 or nine digits in length
//
int add_adv_ZIP_check_digit(char *code_plus, char *addr)
{
  char *isspace,*jzip;
  int  i, zip_sum=0, check_digit, adv_code=0;

  // ALLOCATE A JUNK CHAR POINTER
  jzip = (char *)malloc(strlen(code_plus)+1);

  // TRUNCATE TRAILING SPACES.  EMBEDDED SPACES ARE ALSO BAD
  // SO WE USE strchr INSTEAD OF GOING TO THE END AND COUNTING BACK
  if ((isspace=strchr(code_plus,' ')) != NULL) // truncate trailing spaces
    *isspace = '\0';

  // CHECK TO SEE IF POSSIBLY TRUNCATED ZIP IS WITHIN RANGE
  if (strlen(code_plus) < 5 || strlen(code_plus) > 9)
  {
    free(jzip);
    return(1);
  }

  // ONLY 9 DIGIT ZIP CAN TRY FOR THE 2 ADVANCED DIGITS
  if (strlen(code_plus) == 9 && addr != NULLP)
  {
    for (i=0 ; !isdigit(addr[i]) ; i++);        // find number (if any else 0)
    if (i < strlen(addr))
    {
      adv_code = atoi(&addr[i]);                // make it an integer
      adv_code %= 100;                          // isolate last two digits
      sprintf(jzip,"%s%02d",code_plus,adv_code);// combine with zip code
    }
    else
      strcpy(jzip,code_plus);
  }
  else
    strcpy(jzip,code_plus);

  // CALCULATE THE CHECK DIGIT
  for (i = 0; jzip[i] ; i++)                  // sum all numbers
    zip_sum += jzip[i]-0x30;
  check_digit = zip_sum % 10 == 0 ? 0 : 10-(zip_sum % 10); // compute check

  // FORM THE COMPLETE ZIP BY ADDING THE START AND STOP CODES
  sprintf(code_plus,"*%s%d*",jzip,check_digit);// get it all together

  // TIDY UP
  free(jzip);
  return(0);
}

// A D D _ A S T E R I S K S
//
// The paramter code will have enough spaces to hold the asterisks
//
void add_asterisks(char *code)
{
  char *junk;

  // ALLOCATE A TEMPORARY CHAR ARRAY
  junk = (char *)malloc(strlen(code)+1);
  strcpy(junk,code);
  sprintf(code,"*%s*",junk);
  free(junk);
}

// H P _ G O T O X Y
//
void hp_gotoxy(FILE *fp, int col, int row, int *px, int *py)
{
  // CONVERT FROM COL, ROW TO POINTS, THEN POSITION CURSOR
  *px = (int)((float)col/(float)CPI*DENSITY);
  *py = (int)((float)row/(float)RPI*DENSITY);
  fprintf(fp,"\33*p%dx%dY",*px,*py);
}


